/* $Id: driver_cf.c,v 1.8 2005/04/30 14:24:45 dijkstra Exp $ */

/*
 * Copyright (c) 2003-2005 Willem Dijkstra
 * All rights reserved.
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 *
 *    - Redistributions of source code must retain the above copyright
 *      notice, this list of conditions and the following disclaimer.
 *    - Redistributions in binary form must reproduce the above
 *      copyright notice, this list of conditions and the following
 *      disclaimer in the documentation and/or other materials provided
 *      with the distribution.
 *
 * THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS
 * "AS IS" AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT
 * LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS
 * FOR A PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL THE
 * COPYRIGHT HOLDERS OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT,
 * INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING,
 * BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES;
 * LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER
 * CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT
 * LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN
 * ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE
 * POSSIBILITY OF SUCH DAMAGE.
 *
 */

#if defined (CF632) || defined (CF634) || defined (CF634USB)
#include <fcntl.h>
#include <termios.h>
#include <stdarg.h>
#include <stdio.h>
#include <string.h>
#include <errno.h>

#include "lcd.h"
#include "error.h"
#include "driver.h"

__BEGIN_DECLS
void setup_block();
void setup_vbar();
__END_DECLS

char *driver_name = "Crystalfontz";
int   driver_capabilities = DC_BACKLIGHT | DC_BLOCK | DC_CONTRAST | DC_VBAR;
int   driver_fd = 0;

void
driver_connect(char *serial, int capabilities)
{
    struct termios tp;

    driver_capabilities = DC_CONTRAST | DC_BACKLIGHT;

    if (capabilities & DC_HBAR) {
	fatal("%s: cannot support horizontal bars",
	      driver_name);
    }

    if ((driver_fd = open(serial, O_RDWR | O_NONBLOCK)) == -1) {
	fatal("%s: cannot open serial '%s': %.200s",
	      driver_name, serial, strerror(errno));
    }

    if (tcgetattr(driver_fd, &tp) == -1) {
	fatal("%s: cannot get terminal attributes: %.200s",
	      driver_name, strerror(errno));
    }

    if (cfsetospeed(&tp, LCD_OSPEED) == -1) {
	fatal("%s: cannot set output speed: %.200s",
	      driver_name, strerror(errno));
    }

    if (cfsetispeed(&tp, B0) == -1) {
	fatal("%s: cannot set input speed: %.200s",
	      driver_name, strerror(errno));
    }

    tp.c_iflag &= ~(IGNBRK | BRKINT | PARMRK | ISTRIP | INLCR | IGNCR | ICRNL | IXON);
    tp.c_oflag &= ~OPOST;
    tp.c_lflag &= ~(ECHO | ECHONL | ICANON | ISIG | IEXTEN);
    tp.c_cflag &= ~(CSIZE | PARENB | CRTSCTS);
    tp.c_cflag |= CS8 | CREAD | CLOCAL;
    tp.c_cc[VMIN] = 1;
    tp.c_cc[VTIME] = 3;

    if (tcsetattr(driver_fd, TCSANOW, &tp) == -1) {
	fatal("%s: cannot not set serial line %s to raw: %.200s",
	      driver_name, serial, strerror(errno));
    }

    if (capabilities & DC_BLOCK)
	setup_block();

    if (capabilities & DC_VBAR) {
	setup_block();
	setup_vbar();
    }
}
char
driver_get_hchar(int percentage)
{
    fatal("%s: horizontal bars not implemented", driver_name);

    return 0; /* NOT REACHED */
}
char
driver_get_vchar(int percentage)
{
    int unit = 1000 / 8;
    int i;

    percentage %= 101;
    percentage += 5;
    percentage *= 10;

    i = percentage / unit;

    if (i == 0) {
	return CF_CHAR_SPACE;
    }

    if (i == 8) {
	return CF_CHAR_BLOCK;
    }

    return CF_CHAR_VBAR + i - 1;
}
void
driver_nprintf(int maxsize, char *fmt, ...)
{
    va_list args;
    char buf[LCD_MAXSTR];
    int size;

    maxsize %= sizeof(buf);

    va_start(args, fmt);
    vsnprintf(buf, maxsize, fmt, args);
    va_end(args);
    size = strlen(buf);

    write(driver_fd, buf, size);
}
void
driver_printf(char *fmt, ...)
{
    va_list args;
    char buf[LCD_MAXSTR];
    int size;

    va_start(args, fmt);
    size = vsnprintf(buf, sizeof(buf), fmt, args);
    va_end(args);

    write(driver_fd, buf, size);
}
void
driver_reset()
{
    driver_set_contrast(CF_DEFAULT_CONTRAST);
    driver_set_backlight(CF_DEFAULT_BACKLIGHT);

    driver_printf("%c%c%c%c%c%c",
		  CFCMD_HIDE_DISPLAY,
		  CFCMD_HIDE_CURSOR,
		  CFCMD_FORM_FEED,
		  CFCMD_SCROLL_OFF,
		  CFCMD_WRAP_OFF,
		  CFCMD_RESTORE_DISPLAY);
}
void
driver_set_backlight(int backlight)
{
    driver_printf("%c%c",
		  CFCMD_BACKLIGHT_CONTROL,
		  backlight % 101);
}
void
driver_set_contrast(int contrast)
{
    driver_printf("%c%c",
		  CFCMD_CONTRAST_CONTROL,
		  contrast % 101);
}
void
driver_set_specialchar(int ch, struct specialchar *data)
{
    int ch_data[8];
    int i, bit, mask;

    if ((ch < CF_CHAR_SPECIAL) || (ch > CF_CHAR_SPECIAL + 7)) {
	fatal("%s: cannot set specialchar %d",
	      driver_name, ch);
    }

    for (i = 0; i < 8; i ++) {
	ch_data[i] = 0;
	mask = 0xF;

	for (bit = 0; bit < 6; bit ++) {
	    ch_data[i] += ((data->line[i] & mask) ? (1 << bit) : 0);
	    mask <<= 4;
	}
    }

    driver_printf("%c%c%c%c%c%c%c%c%c%c",
		  CFCMD_SET_CUSTOM_CHARACTER_BITMAP,
		  ch - CF_CHAR_SPECIAL,
		  ch_data[0], ch_data[1], ch_data[2], ch_data[3],
		  ch_data[4], ch_data[5], ch_data[6], ch_data[7]);
}
void
driver_set_xy(int x, int y)
{
    driver_printf("%c%c%c",
		  CFCMD_SET_CURSOR_POSITION,
		  x % LCD_X, y % LCD_Y);
}
void
setup_block()
{
    struct specialchar block = {
	{
	    0x111111,
	    0x111111,
	    0x111111,
	    0x111111,
	    0x111111,
	    0x111111,
	    0x111111,
	    0x111111,
	}
    };

    driver_set_specialchar(CF_CHAR_BLOCK, &block);
    driver_capabilities |= DC_BLOCK;
}
void
setup_vbar()
{
    int i;
    struct specialchar vbar[] = {
	{{
	    0x000000,
	    0x000000,
	    0x000000,
	    0x000000,
	    0x000000,
	    0x000000,
	    0x000000,
	    0x111111,
	}},
	{{
	    0x000000,
	    0x000000,
	    0x000000,
	    0x000000,
	    0x000000,
	    0x000000,
	    0x111111,
	    0x111111,
	}},
	{{
	    0x000000,
	    0x000000,
	    0x000000,
	    0x000000,
	    0x000000,
	    0x111111,
	    0x111111,
	    0x111111,
	}},
	{{
	    0x000000,
	    0x000000,
	    0x000000,
	    0x000000,
	    0x111111,
	    0x111111,
	    0x111111,
	    0x111111,
	}},
	{{
	    0x000000,
	    0x000000,
	    0x000000,
	    0x111111,
	    0x111111,
	    0x111111,
	    0x111111,
	    0x111111,
	}},
	{{
	    0x000000,
	    0x000000,
	    0x111111,
	    0x111111,
	    0x111111,
	    0x111111,
	    0x111111,
	    0x111111,
	}},
	{{
	    0x000000,
	    0x111111,
	    0x111111,
	    0x111111,
	    0x111111,
	    0x111111,
	    0x111111,
	    0x111111,
	}}
    };

    for (i = 0; i < (sizeof(vbar) / sizeof(struct specialchar)); i++) {
	driver_set_specialchar(CF_CHAR_VBAR + i, &vbar[i]);
    }

    driver_capabilities &= !DC_HBAR;
    driver_capabilities |= DC_VBAR;
}
#endif
